@@ -5,8 +5,9 @@ import glob
 import subprocess
 from subprocess import Popen
 
-from PySide2.QtWidgets import QApplication, QMainWindow, QHBoxLayout, QWidget, QFileDialog, QMessageBox, QSplitter, QSizePolicy, QVBoxLayout, QProgressDialog
-from PySide2.QtCore import Slot, Qt, QSettings
+from PySide2.QtWidgets import QApplication, QMainWindow, QHBoxLayout, QWidget, QFileDialog, QMessageBox, QSplitter, QSizePolicy, QVBoxLayout, QProgressDialog, QAction, QToolBar, QDialog, QDialogButtonBox, QLabel, QLineEdit, QDoubleSpinBox
+from PySide2.QtCore import Slot, Qt, QSettings, Signal, QThread # QRunnable, QThreadPool
+from PySide2.QtGui import QIcon
 
 from .transcripts.transcripts import Transcripts
 from .minutes.minutes import Minutes
@@ -15,8 +16,13 @@ from .annotation import Annotation
 from .problems import Problems
 from .settings import Settings
 from .evaluation import Evaluation
+from .autoalign import Aligner
 
 class Annotations(QMainWindow):
+    # main app window class 
+    # creates menus and widgets, puts them in layout
+    # has slots for opening folder/repo, saving
+    # shows discard dialog box
 
     def __init__(self, *args, **kwargs):
         super(Annotations, self).__init__(*args, **kwargs)
@@ -26,6 +32,7 @@ class Annotations(QMainWindow):
         self._gui_setup()
         self.setAttribute(Qt.WA_DeleteOnClose)
         self.is_git = False
+        self.new_problem = ''
 
     def _gui_setup(self):
         layout = QHBoxLayout()
@@ -34,8 +41,10 @@ class Annotations(QMainWindow):
         panel_right = QWidget(self)
         layout.addWidget(splitter)
 
-        annotation = Annotation()
+        annotation = Annotation(self)
         self.annotation = annotation
+        self.annotation.undo_stack.canRedoChanged.connect(self._redo_toggle)
+        self.annotation.undo_stack.canUndoChanged.connect(self._undo_toggle)
 
         transcripts = Transcripts(annotation, self)
         transcripts.setDisabled(True)
@@ -51,11 +60,12 @@ class Annotations(QMainWindow):
         minutes.setDisabled(True)
         self.minutes = minutes
 
-        problems = Problems(self)
+        problems = Problems(annotation, self)
         problems.setEnabled(False)
         problems.problem_selected.connect(self.annotation.set_problem)
         problems.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
         self.problems = problems
+        annotation.problems_changed.connect(problems.refresh)
         
         splitter_right = QSplitter(Qt.Vertical)
         splitter_right.addWidget(minutes)
@@ -76,50 +86,121 @@ class Annotations(QMainWindow):
         widget.setLayout(layout)
         self.setCentralWidget(widget)
 
-        menu = self.menuBar()
-        file_menu = menu.addMenu('&File')
-
-        a = file_menu.addAction('&New')
-        a.setShortcut('Ctrl+n')
-        a.triggered.connect(self.new)
-
-        a = file_menu.addAction('&Open existing')
-        a.setShortcut('Ctrl+o')
-        a.triggered.connect(self.open_existing)
+        # create all menu and toolbar actions
+        self.newAction              = self._createAction('&New', 'Ctrl+n', self.new)
+        self.openMeetingAction      = self._createAction('&Open meeting', 'Ctrl+o', self.open_existing)
+        self.openRepositoryAction   = self._createAction('&Open repository', 'Ctrl+g', self.open_repository)
+        self.evalModeAction         = self._createAction('Evaluation mode', 'Ctrl+e', self.set_evaluation_mode)
+        self.evalModeAction.setCheckable(True)
+        self.saveAction             = self._createAction('&Save', 'Ctrl+s', self.save)
+        self.saveAction.setEnabled(False)
+        self.settingsAction         = self._createAction('S&ettings', 'Ctrl++', lambda : Settings(self).exec_())
+        self.closeAction            = self._createAction('&Close', 'Alt+c', self.close)
+        self.undoAction             = self._createAction('&Undo', 'Ctrl+z', self.annotation.undo)
+        self.undoAction.setIcon(QIcon(os.path.join(os.path.dirname(__file__),"icons/arrow-return-180-left.png")))
+        self.undoAction.setEnabled(False)
+        self.annotation.undo_toggle.connect(self._undo_toggle)
+        self.redoAction             = self._createAction('&Redo', 'Ctrl+y', self.annotation.redo)
+        self.redoAction.setIcon(QIcon(os.path.join(os.path.dirname(__file__),"icons/arrow-return.png")))
+        self.redoAction.setEnabled(False)
+        self.annotation.redo_toggle.connect(self._redo_toggle)
+        self.editHistoryAction      = self._createAction('Show edit &history', 'Alt+h', self.annotation.show_edit_history)
+        self.openAudioAction        = self._createAction('&Open audio', 'Alt+o', self._open_audio)
+        self.autoalignAction        = self._createAction('&Autoalign', 'Alt+a', self._autoalign)
+        self.autoalignAction.setEnabled(False)
+        self.aaFinalizeAction       = self._createAction('&Finalize', 'Alt+f', self._aaFinalize)
+        self.aaFinalizeAction.setEnabled(False)
+        self.aaFinalizeAllAction    = self._createAction('Fi&nalize all', '', self._aaFinalizeAll)
+        self.aaFinalizeAllAction.setEnabled(False)
         
-        a = file_menu.addAction('&Open repository')
-        a.setShortcut('Ctrl+g')
-        a.triggered.connect(self.open_repository)
+        # create menu bar
+        menu = self.menuBar()
 
+        file_menu = menu.addMenu('&File')
+        file_menu.addAction(self.newAction)
+        file_menu.addAction(self.openMeetingAction)
+        file_menu.addAction(self.openRepositoryAction)
         file_menu.addSeparator()
 
-        a = file_menu.addAction('Evaluation mode')
-        a.setCheckable(True)
-        a.setShortcut('Ctrl+e')
-        a.triggered.connect(self.set_evaluation_mode)
-
+        file_menu.addAction(self.evalModeAction)
         file_menu.addSeparator()
 
-        a = file_menu.addAction('&Save')
-        a.setShortcut('Ctrl+s')
-        a.triggered.connect(self.save)
+        file_menu.addAction(self.saveAction)
         file_menu.addSeparator()
 
-        a = file_menu.addAction('S&ettings')
-        a.setShortcut('Ctrl++')
-        a.triggered.connect(lambda : Settings(self).exec_())
+        file_menu.addAction(self.settingsAction)
         file_menu.addSeparator()
 
-        a = file_menu.addAction('&Close')
-        a.setShortcut('Alt+c')
-        a.triggered.connect(self.close)
+        file_menu.addAction(self.closeAction)
+
+        edit_menu = menu.addMenu('&Edit')
+        edit_menu.addAction(self.undoAction)
+        edit_menu.addAction(self.redoAction)
+        edit_menu.addAction(self.editHistoryAction)
+
+        edit_menu.addSeparator()
+
+        edit_menu.addAction(self.problems.addProblemAction)
+        edit_menu.addAction(self.problems.refreshProblemsAction)
+
+        edit_menu.addSeparator()
+
+        transcript_edit_menu = edit_menu.addMenu('&Transcript')
+
+        transcript_edit_menu.addAction(self.minutes.insertingAction)
+        transcript_edit_menu.addAction(self.minutes.deleteAction)
+        transcript_edit_menu.addAction(self.minutes.joinDownAction)
+        transcript_edit_menu.addAction(self.minutes.joinUpAction)
+        transcript_edit_menu.addAction(self.minutes.splitAction)
+
+        minutes_edit_menu = edit_menu.addMenu('&Minutes')
+
+        minutes_edit_menu.addAction(self.transcripts.insertingAction)
+        minutes_edit_menu.addAction(self.transcripts.deleteAction)
+        minutes_edit_menu.addAction(self.transcripts.joinDownAction)
+        minutes_edit_menu.addAction(self.transcripts.joinUpAction)
+        minutes_edit_menu.addAction(self.transcripts.splitAction)
 
         playback_menu = menu.addMenu('&Playback')
-        a = playback_menu.addAction('&Open audio')
-        a.triggered.connect(self._open_audio)
+        playback_menu.addAction(self.openAudioAction)
         playback_menu.addSeparator()
         playback_menu.addActions(player.playback_actions)
+        
+        autoalign_menu = menu.addMenu('&Autoalign')
+        autoalign_menu.addAction(self.autoalignAction)
+        autoalign_menu.addSeparator()
+        autoalign_menu.addAction(self.aaFinalizeAction)
+        autoalign_menu.addAction(self.aaFinalizeAllAction)        
 
+        # Add toolbar (with undo redo, edit mode toggle)
+        toolbar = QToolBar("Main Toolbar")
+        toolbar.setMovable(False)
+        self.addToolBar(toolbar)
+        toolbar.addAction(self.undoAction)
+        toolbar.addAction(self.redoAction)
+        toolbar.addSeparator()
+        toolbar.addAction(self.evalModeAction)
+        
+        toolbar.addSeparator()
+        toolbar.addAction(self.autoalignAction)
+        
+        thr = QDoubleSpinBox(self)
+        thr.setMinimum(0)
+        thr.setMaximum(1)
+        thr.setDecimals(1)
+        thr.setSingleStep(0.1)
+        thr.setValue(0.5)
+        thr.valueChanged.connect(self._setThr)
+        
+        toolbar.addWidget(thr)
+        
+        toolbar.addSeparator()
+        toolbar.addAction(self.aaFinalizeAction)
+        toolbar.addAction(self.aaFinalizeAllAction)
+        
+        self.toolbar = toolbar
+
+        # Set up settings for saving layout upon close
         s = QSettings(self)
         self.splitter = splitter
         self.splitter_righ = splitter_right
@@ -131,6 +212,12 @@ class Annotations(QMainWindow):
         splitter_right.setStretchFactor(0, 10)
         splitter_right.setStretchFactor(1, 1)
 
+    def _createAction(self, name : str, shortcut : str, connection):
+        a = QAction(name)
+        a.setShortcut(shortcut)
+        a.triggered.connect(connection)
+        return a
+
     def _discard_dialog(self):
         msg = QMessageBox(self)
         msg.setIcon(QMessageBox.Warning)
@@ -255,6 +342,12 @@ class Annotations(QMainWindow):
                 os.mkdir(m)
                 io.open(os.path.join(m,'minutes.txt'),'w').write('')
                 io.open(os.path.join(t,'transcript.txt'),'w').write('')
+                self.saveAction.setEnabled(True)
+                self.autoalignAction.setEnabled(True)
+                self.aaFinalizeAction.setEnabled(True)
+                self.aaFinalizeAllAction.setEnabled(True)
+                self.problems.addProblemAction.setEnabled(True)
+                self.problems.refreshProblemsAction.setEnabled(True)
             if self._check_path(path):
                 self.annotation.set_path(path)
                 self.transcripts.setEnabled(True)
@@ -270,6 +363,12 @@ class Annotations(QMainWindow):
                             if self.player.open_audio(os.path.normpath(os.path.join(path, f))):
                                 break
                 self.is_git = False
+                self.saveAction.setEnabled(True)
+                self.aaFinalizeAction.setEnabled(True)
+                self.aaFinalizeAllAction.setEnabled(True)
+                self.problems.addProblemAction.setEnabled(True)
+                self.problems.refreshProblemsAction.setEnabled(True)
+                
                 return True
             else:
                 msg = QMessageBox()
@@ -333,6 +432,14 @@ class Annotations(QMainWindow):
                 file = dlg.selectedFiles()[0]
                 self.player.open_audio(file)
 
+    @Slot(bool)
+    def _undo_toggle(self, value):
+        self.undoAction.setEnabled(value)
+
+    @Slot(bool)
+    def _redo_toggle(self, value):
+        self.redoAction.setEnabled(value)
+
     def closeEvent(self, event):
         s = QSettings(self)
         s.setValue('window_state', self.saveState())
@@ -351,7 +458,27 @@ class Annotations(QMainWindow):
             if msg.exec_():
                 if QMessageBox.Save == msg.result():
                     self.save()
+                    self.annotation.undo_view.close()
                 elif QMessageBox.Cancel == msg.result():
                     event.ignore()
+                self.annotation.undo_view.close()
                 return
+
+        self.annotation.undo_view.close()
         return super().closeEvent(event)
+    
+    @Slot(int)
+    def _setThr(self, thr):
+        self.annotation.threshold = thr
+    
+    @Slot()
+    def _autoalign(self):
+        self.annotation.autoalign()
+    
+    @Slot()
+    def _aaFinalize(self):
+        self.annotation.finalize()
+
+    @Slot()
+    def _aaFinalizeAll(self):
+        self.annotation.finalizeAll()
